/*******************************************************************************
 * @file                segBotController.c
 * @brief               Controller code for segBot
 * @date                December 7th, 2014
 * @authors             Alexander Volkov <av333@cornell.edu>,
 *                      Kevin Bonett     <kb476@cornell.edu>,
 *                      Andre Heil       <avh34@cornell.edu>
 * @version             1.0
 *
 * This program uses the is a simple controller that will continuously read
 * from the Wii Nunchuk and transmit a command based on the Nunchuk status.
 *
 * This code has never been used.
 *
 ******************************************************************************/


/*** Program Macros ***********************************************************/

#define F_CPU 16000000UL

#define PIPE_ADDRESS_0     0xB4B5B600


/*** Include Files ************************************************************/

#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "../lib/wiiNunchuk/wiiNunchuk_lib.h"
#include "../lib/wiiNunchuk/wiiNunchuk_lib.c"

#include "../lib/RF/nRF24L01_lib.h"


/*** Program Variables ********************************************************/

int joyX = 0;
int joyY = 0;

char  tx_result;
char  data;
char* packet;


/*** Program Function Prototypes **********************************************/

int  main       (void);
void initialize (void);


/*** Program Functions ********************************************************/

int main(void)
{
	initialize();

	while(1)
	{
		joyX = wiiNunchuk_getJoyX();
		joyY = wiiNunchuk_getJoyY();

		if (joyX > 220) {
			data = "R";
		}
		else if (joyX < 30) {
			data = "L";
		}
		else if (joyY > 220) {
			data = "F";
		}
		else if (joyY < 30) {
			data = "B";
		}
		else {
			data = "N";
		}

		strncpy(packet, data, PAYLOAD_LENGTH);
		transmit_packet(packet);

		wiiNunchuk_update();
	}
}

void initialize(void)
{
	wiiNunchuk_init();

	PAYLOAD_LENGTH = 8;
	init_RF(0);

	set_mode(TX_MODE);

	set_RX_addr(PIPE_ADDRESS_0, 0);
	set_TX_addr(PIPE_ADDRESS_0);

	packet = (char *) malloc(PAYLOAD_LENGTH);
}

