/*******************************************************************************
 * @file                nRF24L01_cmdList.h
 * @brief               A list of command definitions
 * @date                December 3rd, 2014
 * @authors             Alexander Volkov <av333@cornell.edu>,
 *                      Kevin Bonett     <kb476@cornell.edu>,
 *                      Andre Heil       <avh34@cornell.edu>,
 *                      Yi Heng Lee,
 *                      Jaehyun Kim
 * @version             2.0
 *
 * Contains definitions for all the commands used in the chip.
 *
 ******************************************************************************/


/*** Program Macros ***********************************************************/

/* Read command and status registers */
#define R_REGISTER         0b00000000

/* Write command and status registers, Executable in power down or standby mode
 * only */
#define W_REGISTER         0b00100000

/* Read RX payload, starts from LSByte first */
#define R_RX_PAYLOAD       0b01100001

/* Write TX payload, starts from LSByte first */
#define W_TX_PAYLOAD       0b10100000

/* Flush TX FIFO */
#define FLUSH_TX           0b11100001

/* Flush RX_FIFO. Should not be executed during transmission of acknoweledge */
#define FLUSH_RX           0b11100010

/* Reuse last transmitted payload. Active untill W_TX_PAYLOAD or FLUSH_TX is
 * executed. Not to be used during packet transmission. */
#define REUSE_TX_PL        0b11100011

/* Read RX payload width for the top R_RX_PAYLOAD in RX FIFO, flush FIFO if read
 * value is larger than 32 bytes. */
#define R_RX_PL_WID        0b01100000

/* Write payload to be transmitted with ACK packet on PIPE PPP. */
#define W_ACK_PAYLOAD      0b10101000

/* Used in TX mode. Disables AUTOACK on this specific packet. */
#define W_TX_PAYLOAD_NOACK 0b10110000

/* No operation. Used to read status register. */
#define NOP                0b11111111
