/*******************************************************************************
 * @file                nRF24L01_regMap.h
 * @brief               Defines for the nRF24L01+ registers
 * @date                December 4th, 2014
 * @authors             Alexander Volkov <av333@cornell.edu>,
 *                      Kevin Bonett     <kb476@cornell.edu>,
 *                      Andre Heil       <avh34@cornell.edu>,
 *                      Yi Heng Lee,
 *                      Jaehyun Kim
 * @version             2.0
 *
 * Look at nRF24L01+ datasheet pg. 57 for better description of the registers.
 *
 ******************************************************************************/


/*** Program Macros ***********************************************************/

/* Configuration Register:
 *     CRC0:   0 = 1 byte CRC, 1 = 2 byte CRC
 *     EN_CRC: Enables CRC, forced high if one of the bits in EN_AA is high
 */
#define CONFIG 0x00
	#define MASK_RX_DR  6
	#define MASK_TX_DS  5
	#define MASK_MAX_RT 4
	#define EN_CRC      3
	#define CRCO        2
	#define PWR_UP      1
	#define PRIM_RX     0

/* Enhanced Shock Burst:
 *     ENAA_P5: Enable auto acknowledgement, data pipe 5
 */
#define EN_AA 0x01
	#define ENAA_P5 5
	#define ENAA_P4 4
	#define ENAA_P3 3
	#define ENAA_P2 2
	#define ENAA_P1 1
	#define ENAA_P0 0

/* Enabled RX addresses, data pipe */
#define EN_RXADDR 0x02
	#define ERX_P5 5
	#define ERX_P4 4
	#define ERX_P3 3
	#define ERX_P2 2
	#define ERX_P1 1
	#define ERX_P0 0

/* Setup of address width, common for all data pipes */
#define SETUP_AW 0x03

/* Setup Automatic Retransmission:
 *     ARD: Auto retransmit delay 0000 -> 250uS ~ 1111 -> 4000uS
 *     ARC: Auto retransmit count (0 ~ 15)
 */
#define SETUP_RETR 0x04
	#define ARD 4
	#define ARC 0

/* RF Channel Frequency */
#define RF_CH 0x05

/* Setup RF:
 *     RF_DR_LOW: Set RF data rate to 250kBps
 */
#define RF_SETUP 0x06
	#define CONT_WAVE 7
	#define RF_DR_LOW 5
	#define RF_DR_HIGH 3
	#define RF_PWR 1

/* Status Register:
 * In the event of an interrupt, you look at this register to see the source of
 * interrupt, write one to clear bit
 *     RX_P_NO: Which data pipe
 */
#define STATUS 0x07
	#define RX_DR          6
	#define TX_DS          5
	#define MAX_RT         4
	#define RX_P_NO        1
	#define STATUS_TX_FULL 0

/* Transmit Observer */
#define OBSERVE_TX 0x08
	#define PLOS_CNT 4
	#define ARC_CNT 0

/* Received Power Detector */
#define RPD 0x09

/* Receive Address Data Pipe (5 bytes) */
#define RX_ADDR_P0 0x0A
#define RX_ADDR_P1 0x0B
#define RX_ADDR_P2 0x0C
#define RX_ADDR_P3 0x0D
#define RX_ADDR_P4 0x0E
#define RX_ADDR_P5 0x0F

/* Transmit address (5 bytes), used for PTX device only */
#define TX_ADDR 0x10

/* Number of bytes in RX payload in each data pipe */
#define RX_PW_P0 0x11
#define RX_PW_P1 0x12
#define RX_PW_P2 0x13
#define RX_PW_P3 0x14
#define RX_PW_P4 0x15
#define RX_PW_P5 0x16

#define FIFO_STATUS 0x17
	#define TX_REUSE            6
	#define FIFO_STATUS_TX_FULL 5
	#define TX_EMPTY            4
	#define RX_FULL             1
	#define RX_EMPTY            0

/* Enable Dynamic Payload Length */
#define DYNPD 0x1C
	#define DPL_P5 5
	#define DPL_P4 4
	#define DPL_P3 3
	#define DPL_P2 2
	#define DPL_P1 1
	#define DPL_P0 0

#define FEATURE 0x1D
	#define EN_DPL     2
	#define EN_ACK_PAY 1
	#define EN_DYN_ACK 0

