/*******************************************************************************
 * @file                wiiNunchuk_lib.h
 * @brief               wiiNunchuk settings
 * @date                December 2nd, 2014
 * @authors             Alexander Volkov <av333@cornell.edu>,
 *                      Kevin Bonett     <kb476@cornell.edu>,
 *                      Andre Heil       <avh34@cornell.edu>,
 *                      David Gironi
 * @version             2.0
 *
 * This header file controls the settings to wiiNunchuk.c  You can set various
 * settings here such as the calibration, pulse button, I2C settings, etc.
 *
 ******************************************************************************/

#ifndef WIINUNCHUK_H_
#define WIINUNCHUK_H_


/*** Program Macros ***********************************************************/

/* Definitions */
#define WIINUNCHUK_ADDR    (0x52 << 1) // Device address
#define WIINUNCHUK_READBYTES         6

/* Enable a more precise function for centering joypad:
 *     0 to select o static offset center
 *     1 to enable a math function to center the joypad */
#define WIINUNCHUK_JOYCENTERB        1

/* Enable the attitude estimation */
#define WIINUNCHUK_GETATTITUDE       1

/* I2C Settings */
#define WIINUNCHUK_I2CINIT           1

/* Get non calibrated values, use this function to calibrate the device */
#define WIINUNCHUK_GETNONCALIBRATED  0

/* Pulse button enabled */
#define WIINUNCHUK_PULSEBUTTON       0

/* Angle filter enabled */
#define WIINUNCHUK_ANGLEFILTER       1
#define WIINUNCHUK_ANGLEAVERAGECOEF 21 //number of samples to average

/* Calibration */
#define WIINUNCHUK_ZEROANGLEX      527
#define WIINUNCHUK_ZEROANGLEY      531
#define WIINUNCHUK_ZEROANGLEZ      530

#if WIINUNCHUK_JOYCENTERB == 1
#define WIINUNCHUK_CENTERJOYX      137  // The center as read
#define WIINUNCHUK_INMINJOYX         0  // The min as read
#define WIINUNCHUK_INMAXJOYX       255  // The max as read
#define WIINUNCHUK_CENTERJOYY      137  // The center as read
#define WIINUNCHUK_INMINJOYY         0  // The min as read
#define WIINUNCHUK_INMAXJOYY       255  // The max as read

#else
#define WIINUNCHUK_DEFAULTZEROJOYX 137 // The center as read
#define WIINUNCHUK_DEFAULTZEROJOYY 137 // The center as read

#endif

/* Decodebyte is necessary with certain initializations */
#define WIINUNCHUK_DECODEBYTE        0

#if WIINUNCHUK_DECODEBYTE == 1
#define WIINUNCHUK_decode(x) (x ^ 0x17) + 0x17;

#endif


/*** Program Function Prototypes **********************************************/

/* Functions */
extern void    wiiNunchuk_calibrateJoy();
extern void    wiiNunchuk_init();
extern int     wiiNunchuk_getJoyX();
extern int     wiiNunchuk_getJoyY();
extern uint8_t wiiNunchuk_getButtonZ();
extern uint8_t wiiNunchuk_getButtonC();
extern int     wiiNunchuk_getAngleX();
extern int     wiiNunchuk_getAngleY();
extern int     wiiNunchuk_getAngleZ();
extern void    wiiNunchuk_update();

#if WIINUNCHUK_GETATTITUDE == 1
extern void wiiNunchuk_getPitchRoll(double ax, double ay, double az, double *pitch, double *roll);
#endif

#endif
