/*******************************************************************************
 * @file                test_RF_TX.c
 * @brief               Program to test the RF link, RX side
 * @date                December 6th, 2014
 * @authors             Alexander Volkov <av333@cornell.edu>,
 *                      Kevin Bonett     <kb476@cornell.edu>,
 *                      Andre Heil       <avh34@cornell.edu>
 * @version             1.0
 *
 * This program uses the TRT operating system to schedule a task to test the
 * functionality of the nRF24L01+ receiving operations.
 *
 ******************************************************************************/


/*** Program Macros ***********************************************************/

#define PIPE_ADDRESS_0     0xB4B5B600

#define SEM_RX_ISR_SIGNAL  1
#define SEM_STRING_DONE    2

#define UART_BAUD          9600


/*** Include Files ************************************************************/

#include "../../lib/TRT/trtSettings.h"
#include "../../lib/TRT/trtKernel_644.c"

#include <inttypes.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "../../lib/TRT/trtUart.h"
#include "../../lib/TRT/trtUart.c"

#include "../../lib/RF/nRF24L01_lib.h"


/*** Program Variables ********************************************************/

/* UART file desriptor */
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

/* Input arguments to each thread, not actually used. */
int args[3];


/*** Program Function Prototypes **********************************************/

int  main       (void);
void initialize (void);

void testTask   (void *args);


/*** Program Functions ********************************************************/

int main(void)
{
	initialize();

	/* Sets up the kernel data structures. The parameter is the desired starck
	 * size of the idle task. For a null idle task, a stack size of 80 should be
	 * sufficient. */
	trtInitKernel(80);

	/* Identifies the three functions to the kernel as threads. The parameters
	 * specify pointers to the functions, the desired stack size, the initial
	 * release time, the initial deadline time, and an abitrary data input
	 * structure. */
	trtCreateTask(testTask, 200, SECONDS2TICKS(0.5), SECONDS2TICKS(0.5), &(args[0]));

	/* Choose our preferred sleep mode */
	set_sleep_mode(SLEEP_MODE_IDLE);

	/* Set sleep enable (SE) bit */
	sleep_enable();

	/* Sleep the CPU when a task isn't running. */
	while (1) { sleep_cpu(); }
}

void initialize(void)
{
	trt_uart_init();

	stdout = stdin = stderr = &uart_str;
	fprintf(stdout, "TinyRealTime: 2/9/09\n\r");
}

void testTask(void *args)
{
	unsigned char i;
	unsigned char pipe_rx;

	PAYLOAD_LENGTH = 4;
	init_RF(0);

	set_RX_addr(PIPE_ADDRESS_0, 0);

	char* packet = (char *) malloc(PAYLOAD_LENGTH);

	fprintf(stdout, "Starting...\n");
	while (1)
	{
		if (RX_FIFO_status() == 1) {
			pipe_rx = read_RX_payload(packet);

			for (i = 0; i < PAYLOAD_LENGTH; i++) {
				fprintf(stdout, "%c", packet[i]);
			}
		}

		rel  = trtCurrentTime() + SECONDS2TICKS(1.0);
		dead = trtCurrentTime() + SECONDS2TICKS(1.3);
		trtSleepUntil(rel, dead);
	}
}
