#include <avr/io.h>
#include <stdio.h>
#define F_CPU 16000000UL
#include <util/delay.h>
#include <avr/interrupt.h>

#include "nRF24L01.h"

#define UART_BAUD_RATE 9600
#include "uart/uart.h"
#include "uart/uart.c"

#define CE   3
#define CSN  4
#define MOSI 5
#define MISO 6
#define SCLK 7

uint8_t *data;

FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

void SPI_init(void)
{
	DDRB |= BIT(DDB7) | BIT(DDB5) | BIT(DDB4) | BIT(DDB3);
	SPCR |= BIT(SPE)  | BIT(MSTR);

	EICRA = 3;
	EIMSK |=  (BIT(INT0));


	SETBIT(PORTB, CSN);
	CLEARBIT(PORTB, CE);
}

char SPI_W(unsigned char cData)
{
	SPDR = cData;

	while (!(SPSR & (1 << SPIF)));

	return SPDR;
}

char SPI_R()
{
	return SPI_W(NOP);
}

uint8_t nRF_R(uint8_t reg)
{
	_delay_us(10);
	CLEARBIT(PORTB, CSN);

	_delay_us(10);
	SPI_W(R_REGISTER + reg);

	_delay_us(10);
	reg = SPI_R();

	_delay_us(10);
	SETBIT(PORTB, CSN);

	return reg;
}

void nRF_W(uint8_t reg, uint8_t package)
{
	_delay_us(10);
	CLEARBIT(PORTB, CSN);

	_delay_us(10);
	SPI_W(W_REGISTER + reg);

	_delay_us(10);
	SPI_W(package);

	_delay_us(10);
	SETBIT(PORTB, CSN);
}

uint8_t* nRF_RW(uint8_t RW, uint8_t reg, uint8_t *val, uint8_t antVal)
{
	/* RW: (W or R), reg: (the register), *val: (an array with the package), antVal: (number of integers in the package) */
	int i;
	static uint8_t ret[32];

	if (RW == W) {
		reg = W_REGISTER + reg;
	}
	else {
		reg = R_REGISTER + reg;
	}

	_delay_us(10);
	CLEARBIT(PORTB, CSN);

	_delay_us(10);
	SPI_W(reg);

	_delay_us(10);
	for (i = 0; i < antVal; i++) {
		if (RW == R) {
			ret[i] = SPI_R();
			_delay_us(10);
		}
		else {
			SPI_W(val[i]);
			_delay_us(10);
		}
	}

	SETBIT(PORTB, CSN);

	return ret;
}

void nRF_init(void)
{
	uint8_t val[5];
	int i;

	_delay_ms(100);

	nRF_W(EN_AA, 0x01);

	nRF_W(EN_RXADDR, 0x01);

	nRF_W(SETUP_AW, 0x03);

	nRF_W(RF_CH, 0x01);

	nRF_W(RF_SETUP, 0x07);

	for (i = 0; i < 5; i++) {
		val[i] = 0xE7;
	}

	nRF_RW(W, RX_ADDR_P0, val, 5);

	nRF_W(RX_PW_P0, 4);

	nRF_W(CONFIG, BIT(PWR_UP) | BIT(PRIM_RX));

	_delay_ms(100);
}

void nRF_TX(uint8_t *W_buff)
{
	nRF_RW(R, FLUSH_TX, W_buff, 0);
	nRF_RW(W, W_TX_PAYLOAD, W_buff, 5);

	_delay_ms(10);
	SETBIT(PORTB, CE);

	_delay_us(1000);
	CLEARBIT(PORTB, CE);

	_delay_ms(10);
}

void nRF_RX(void)
{
	SETBIT(PORTB, CE);

	_delay_ms(1000);
	CLEARBIT(PORTB, CE);
}

void nRF_reset(void)
{
	_delay_us(10);
	CLEARBIT(PORTB, CSN);

	_delay_us(10);
	SPI_W(W_REGISTER + STATUS);

	_delay_us(10);
	SPI_W(0x70);

	_delay_us(10);
	SETBIT(PORTB, 2);
}

ISR(INT0_vect)
{
	cli();

	data = nRF_RW(R, R_RX_PAYLOAD, data, 5);
	fprintf(stdout, "%s\n", data);

	nRF_reset();

	sei();
}

int main(void)
{
	uint8_t W_buff[5];
	int i;

	SPI_init();
	nRF_init();

	uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout, "UART initialized\n");

	while (1)
	{
		nRF_RX();
	}
}
